@@ -53,18 +53,25 @@ public:
     /**************************************************************************************************
     Get an argument
     **************************************************************************************************/
-    void GetArgument(int nArgument, APE::CSmartPtr<str_utfn> & rspString) const
+    void GetArgument(int nArgument, APE::CSmartPtr<APE::str_utfn> & rspString) const
     {
         if ((nArgument >= 0) && (nArgument < m_argc))
         {
             #ifdef PLATFORM_WINDOWS
                 #ifdef _UNICODE
+                    // make a copy of the string
+                    //rspString.Assign(new APE::str_utfn [wcslen(m_argv[nArgument]) + 1], true);
+                    //wcscpy_s(rspString, wcslen(m_argv[nArgument]) + 1, m_argv[nArgument]);
+
+                    // simply return the string from the buffer and tell the pointer not to delete
                     rspString.Assign(m_argv[nArgument], true, false);
                 #else
+                    // convert from ANSI
                     rspString.Assign(CAPECharacterHelper::GetUTFNFromANSI(m_argv[nArgument]), true);
                 #endif
             #else
-               rspString.Assign(CAPECharacterHelper::GetUTFNFromUTF8((str_utf8 *) m_argv[nArgument]), true);
+                // convert from UTF-8
+                rspString.Assign(CAPECharacterHelper::GetUTFNFromUTF8((str_utf8 *) m_argv[nArgument]), true);
             #endif
         }
         else
@@ -75,22 +82,90 @@ public:
     }
 
     /**************************************************************************************************
-    Get threads argument
+    Get arguments
     **************************************************************************************************/
-    bool GetThreads(int nArgument, int & rnThreads) const
+    void GetOptions(int & rnMode, int & rnCompressionLevel, int & rnThreads, bool & rbReadFullInput, APE::CSmartPtr<APE::str_utfn> & rspTagSwitch, APE::CSmartPtr<APE::str_utfn> & rspInputFilename, APE::CSmartPtr<APE::str_utfn> & rspOutputFilename) const
     {
-        // get the threads argument
-        CSmartPtr<str_utfn> spThreads;
-        GetArgument(nArgument, spThreads);
+        // defaults (these are already set by the caller, but just here for good form)
+        rnMode = UNDEFINED_MODE;
+        rnCompressionLevel = APE_COMPRESSION_LEVEL_NORMAL;
+        rnThreads = 1;
+        rbReadFullInput = false;
+        rspTagSwitch.Delete();
+        rspInputFilename.Delete();
+        rspOutputFilename.Delete();
 
-        bool bFound = false;
-        if (spThreads && (_wcsnicmp(spThreads, L"-threads=", 9) == 0))
+        // load
+        for (int z = 1; ; z++)
         {
-            rnThreads = _wtoi(&spThreads[9]);
-            bFound = true;
+            // get the argument
+            CSmartPtr<str_utfn> spOption;
+            GetArgument(z, spOption);
+            if (spOption == APE_NULL)
+                break; // we reached the end
+
+            // check for known arguments
+            // put full words before short stuff like -r since that can also be in the full words
+            if (_wcsnicmp(spOption, L"-threads=", 9) == 0)
+            {
+                rnThreads = _wtoi(&spOption[9]);
+            }
+            else if (_wcsnicmp(spOption, L"-readfullinput", 14) == 0)
+            {
+                rbReadFullInput = true;
+            }
+            else if (_wcsnicmp(spOption, L"-c", 2) == 0)
+            {
+                rnMode = COMPRESS_MODE;
+                rnCompressionLevel = _wtoi(&spOption[2]);
+            }
+            else if (_wcsnicmp(spOption, L"-d", 2) == 0)
+            {
+                rnMode = DECOMPRESS_MODE;
+            }
+            else if (wcsncmp(spOption, L"-V", 2) == 0)
+            {
+                rnMode = VERIFY_FULL_MODE;
+            }
+            else if (wcsncmp(spOption, L"-v", 2) == 0)
+            {
+                rnMode = VERIFY_MODE;
+            }
+            else if (_wcsnicmp(spOption, L"-n", 2) == 0)
+            {
+                rnMode = CONVERT_MODE;
+                rnCompressionLevel = _wtoi(&spOption[2]);
+            }
+            else if (_wcsnicmp(spOption, L"-r", 2) == 0)
+            {
+                rnMode = REMOVE_TAG_MODE;
+            }
+            else if (_wcsnicmp(spOption, L"-t", 2) == 0)
+            {
+                if (rnMode == UNDEFINED_MODE)
+                    rnMode = TAG_MODE;
+                GetArgument(z + 1, rspTagSwitch);
+                z++;
+            }
+            else if (_wcsnicmp(spOption, L"-L", 2) == 0)
+            {
+                rnMode = CONVERT_TAG_TO_LEGACY;
+            }
+            else if (rspInputFilename == APE_NULL)
+            {
+                GetArgument(z, rspInputFilename);
+            }
+            else if (rspOutputFilename == APE_NULL)
+            {
+                GetArgument(z, rspOutputFilename);
+            }
         }
 
-        return bFound;
+        // validate
+        if ((rspInputFilename != APE_NULL) && (FileExists(rspInputFilename) == false))
+        {
+            rspInputFilename.Delete();
+        }
     }
 };
 
@@ -153,6 +228,7 @@ static void DisplayProperUsage(FILE * pFile)
 
     fwprintf(pFile, L"Options:\n");
     fwprintf(pFile, L"    Set the number of threads when compressing or decompressing: -threads=#'\n\n");
+    fwprintf(pFile, L"    Read the entire file until the end of file if it's unknown length so the seek table can be made perfect: -readfullinput'\n\n");
 
     fwprintf(pFile, L"Examples:\n");
     fwprintf(pFile, L"    Compress: mac.exe \"Metallica - One.wav\" \"Metallica - One.ape\" -c2000\n");
@@ -440,10 +516,7 @@ int _tmain(int argc, TCHAR * argv[])
 #endif
 {
     // variable declares
-    CSmartPtr<str_utfn> spInputFilename; CSmartPtr<str_utfn> spOutputFilename;
     int nRetVal = ERROR_UNDEFINED;
-    int nMode = UNDEFINED_MODE;
-    int nCompressionLevel = 0;
     int nPercentageDone;
 
     // initialize
@@ -458,64 +531,30 @@ int _tmain(int argc, TCHAR * argv[])
     // output the header
     fwprintf(stderr, CONSOLE_NAME);
 
-    // make sure there are at least four arguments (could be more for EAC compatibility)
+    // make sure there are at least four arguments
     if (argc < 3)
     {
         DisplayProperUsage(stderr);
         exit(-1);
     }
 
-    // store the filenames
-    Arguments.GetArgument(1, spInputFilename);
-    Arguments.GetArgument(2, spOutputFilename);
+    // get the arguments
+    int nMode = UNDEFINED_MODE;
+    int nCompressionLevel = APE_COMPRESSION_LEVEL_NORMAL;
+    int nThreads = 1;
+    bool bReadFullInput = false;
+    APE::CSmartPtr<APE::str_utfn> spTagSwitch;
+    APE::CSmartPtr<APE::str_utfn> spInputFilename;
+    APE::CSmartPtr<APE::str_utfn> spOutputFilename;
+    Arguments.GetOptions(nMode, nCompressionLevel, nThreads, bReadFullInput, spTagSwitch, spInputFilename, spOutputFilename);
 
-    // verify that the input file exists
-    if (!FileExists(spInputFilename))
+    // verify that the input file exists (we check existence when we parse the options)
+    if (spInputFilename == APE_NULL)
     {
         fwprintf(stderr, L"Input file not found...\n\n");
         exit(-1);
     }
 
-    // get the mode argument
-    CSmartPtr<str_utfn> spMode;
-    Arguments.GetArgument(2, spMode);
-
-    if ((wcsncmp(spMode, L"-v", 2) != 0) &&
-        (wcsncmp(spMode, L"-V", 2) != 0) &&
-        (_wcsnicmp(spMode, L"-t", 2) != 0) &&
-        (_wcsnicmp(spMode, L"-r", 2) != 0) &&
-        (_wcsnicmp(spMode, L"-L", 2) != 0))
-    {
-        // modes other than verify, tag, remove tag, and legacy use at least the third argument
-        if (argc < 4)
-        {
-            DisplayProperUsage(stderr);
-            exit(-1);
-        }
-
-        // get the mode (since it will be the next argument in these cases)
-        Arguments.GetArgument(3, spMode);
-    }
-
-    // get the mode
-    nMode = UNDEFINED_MODE;
-    if (_wcsnicmp(spMode, L"-c", 2) == 0)
-        nMode = COMPRESS_MODE;
-    else if (_wcsnicmp(spMode, L"-d", 2) == 0)
-        nMode = DECOMPRESS_MODE;
-    else if (wcsncmp(spMode, L"-V", 2) == 0)
-        nMode = VERIFY_FULL_MODE;
-    else if (wcsncmp(spMode, L"-v", 2) == 0)
-        nMode = VERIFY_MODE;
-    else if (_wcsnicmp(spMode, L"-n", 2) == 0)
-        nMode = CONVERT_MODE;
-    else if (_wcsnicmp(spMode, L"-r", 2) == 0)
-        nMode = REMOVE_TAG_MODE;
-    else if (_wcsnicmp(spMode, L"-t", 2) == 0)
-        nMode = TAG_MODE;
-    else if (_wcsnicmp(spMode, L"-L", 2) == 0)
-        nMode = CONVERT_TAG_TO_LEGACY;
-
     // error check the mode
     if (nMode == UNDEFINED_MODE)
     {
@@ -523,33 +562,14 @@ int _tmain(int argc, TCHAR * argv[])
         exit(-1);
     }
 
-    // get threads
-    int nThreads = 1;
-    int nTagArgument = 5;
-    if (nMode == COMPRESS_MODE)
-    {
-        // get the threads argument
-        if (Arguments.GetThreads(4, nThreads))
-            nTagArgument = 6;
-    }
-    else if (nMode == DECOMPRESS_MODE)
-    {
-        // get the threads argument
-        Arguments.GetThreads(4, nThreads);
-    }
-    else if ((nMode == VERIFY_FULL_MODE) || (nMode == VERIFY_MODE))
-    {
-        // get the threads argument
-        Arguments.GetThreads(3, nThreads);
-    }
-
     // get and error check the compression level
     if (nMode == COMPRESS_MODE || nMode == CONVERT_MODE)
     {
-        nCompressionLevel = _wtoi(&spMode[2]);
-        if (nCompressionLevel != 1000 && nCompressionLevel != 2000 &&
-            nCompressionLevel != 3000 && nCompressionLevel != 4000 &&
-            nCompressionLevel != 5000)
+        if (nCompressionLevel != APE_COMPRESSION_LEVEL_FAST &&
+            nCompressionLevel != APE_COMPRESSION_LEVEL_NORMAL &&
+            nCompressionLevel != APE_COMPRESSION_LEVEL_HIGH &&
+            nCompressionLevel != APE_COMPRESSION_LEVEL_EXTRA_HIGH &&
+            nCompressionLevel != APE_COMPRESSION_LEVEL_INSANE)
         {
             DisplayProperUsage(stderr);
             return ERROR_UNDEFINED;
@@ -573,22 +593,14 @@ int _tmain(int argc, TCHAR * argv[])
         HandleAuto(spInputFilename, spOutputFilename, true);
 
         // compress
-        nRetVal = CompressFileW(spInputFilename, spOutputFilename, nCompressionLevel, &nPercentageDone, ProgressCallback, &nKillFlag, nThreads);
+        nRetVal = CompressFileW(spInputFilename, spOutputFilename, nCompressionLevel, &nPercentageDone, ProgressCallback, &nKillFlag, nThreads, APE_NULL, bReadFullInput);
 
         // tag
-        if ((nRetVal == ERROR_SUCCESS) && (argc > nTagArgument))
+        if ((nRetVal == ERROR_SUCCESS) && (spTagSwitch != APE_NULL))
         {
-            APE::CSmartPtr<str_utfn> spTagSwitch;
-            Arguments.GetArgument(nTagArgument - 1, spTagSwitch);
-            if ((spTagSwitch.GetPtr() != NULL) && (_wcsicmp(spTagSwitch, L"-t") == 0))
-            {
-                fwprintf(stderr, L"\nTagging...\n");
+            fwprintf(stderr, L"\nTagging...\n");
 
-                CSmartPtr<str_utfn> spTag;
-                Arguments.GetArgument(nTagArgument, spTag);
-
-                nRetVal = Tag(spOutputFilename, spTag);
-            }
+            nRetVal = Tag(spOutputFilename, spTagSwitch);
         }
     }
     else
@@ -621,9 +633,7 @@ int _tmain(int argc, TCHAR * argv[])
     else if (nMode == TAG_MODE)
     {
         fwprintf(stderr, L"Tagging...\n");
-        CSmartPtr<str_utfn> spTag;
-        Arguments.GetArgument(3, spTag);
-        nRetVal = Tag(spInputFilename, spTag);
+        nRetVal = Tag(spInputFilename, spTagSwitch);
     }
     else if (nMode == REMOVE_TAG_MODE)
     {
