@@ -1,4 +1,4 @@
-#pragma once
+﻿#pragma once
 
 /**************************************************************************************************
 Platform
@@ -80,6 +80,7 @@ Global includes
 #define APE_MIN(a, b) (((a) < (b)) ? (a) : (b))
 #define APE_CAP(value, low, high) (((value) < (low)) ? (low) : ((value) > (high)) ? (high) : (value))
 #define APE_CLEAR(destination) memset(&destination, 0, sizeof(destination))
+#define APE_CLEAR_ARRAY(destination, elements) memset(&destination[0], 0, sizeof(destination[0]) * static_cast<size_t>(elements))
 
 /**************************************************************************************************
 Version
@@ -100,11 +101,11 @@ Version
 #define APE_VERSION_STRING                              APE_VER_FILE_VERSION_STR
 #define APE_VERSION_NUMBER                              APE_VERSION_MAJOR APE_VERSION_REVISION
 #define APE_NAME                                        _T("Monkey's Audio ") APE_VER_FILE_VERSION_STR
-#define PLUGIN_NAME                                     "Monkey's Audio Player " APE_VER_FILE_VERSION_STR_NARROW
-#define MJ_PLUGIN_NAME                                  _T("APE Plugin (v") APE_VER_FILE_VERSION_STR _T(")")
+#define APE_PLUGIN_NAME                                 "Monkey's Audio Player " APE_VER_FILE_VERSION_STR_NARROW
+#define APE_MC_PLUGIN_NAME                              _T("APE Plugin (v") APE_VER_FILE_VERSION_STR _T(")")
 #define APE_RESOURCE_VERSION_COMMA                      APE_VERSION_MAJOR, APE_VERSION_REVISION, 0, 0
 #define APE_RESOURCE_VERSION_STRING                     APE_VER_FILE_VERSION_STR
-#define APE_RESOURCE_COPYRIGHT                          "Copyright (c) 2000-" STRINGIZE(APE_YEAR) " Matthew T. Ashland"
+#define APE_RESOURCE_COPYRIGHT                          "Copyright © 2000-" STRINGIZE(APE_YEAR) " by Matthew T. Ashland"
 #define CONSOLE_NAME                                    L"--- Monkey's Audio Console Front End (v " APE_VER_FILE_VERSION_STR_WIDE L") (c) Matthew T. Ashland ---\n"
 #define PLUGIN_ABOUT                                    _T("Monkey's Audio Player v") APE_VER_FILE_VERSION_STR _T("\nCopyrighted (c) 2000-") STRINGIZE(APE_YEAR) _T(" by Matthew T. Ashland")
 
@@ -171,8 +172,10 @@ Global macros
 // undefined file size (pipe, etc.)
 #define APE_FILE_SIZE_UNDEFINED -1
 
-#define POINTER_TO_INT64(POINTER) static_cast<APE::int64>(reinterpret_cast<uintptr_t>(POINTER))
+// make a pointer into an int64
+#define APE_POINTER_TO_INT64(POINTER) static_cast<APE::int64>(reinterpret_cast<uintptr_t>(POINTER))
 
+// platform defines
 #if defined(PLATFORM_WINDOWS)
     #define IO_USE_WIN_FILE_IO
     #define DLLEXPORT                                   __declspec(dllexport)
